@@ -7,13 +7,6 @@
 
 namespace a0 {
 
-struct PathGlob : details::CppWrap<a0_pathglob_t> {
-  PathGlob() = default;
-  explicit PathGlob(std::string path_pattern);
-
-  bool match(const std::string& path) const;
-};
-
 struct Discovery : details::CppWrap<a0_discovery_t> {
   Discovery() = default;
   Discovery(
