@@ -47,8 +47,7 @@ PYBIND11_MODULE(alephzero_bindings, m) {
   py::enum_<a0_arena_mode_t>(pyarena, "Mode")
       .value("SHARED", A0_ARENA_MODE_SHARED)
       .value("EXCLUSIVE", A0_ARENA_MODE_EXCLUSIVE)
-      .value("READONLY", A0_ARENA_MODE_READONLY)
-      .export_values();
+      .value("READONLY", A0_ARENA_MODE_READONLY);
 
   pyarena
       .def(py::init([](a0::File file) { return a0::Arena(file); }))
@@ -107,7 +106,7 @@ PYBIND11_MODULE(alephzero_bindings, m) {
       .def(py::init([](py::bytes payload) {
              return a0::Packet(as_string_view(payload), a0::ref);
            }),
-           py::keep_alive<0, 1>())
+           py::keep_alive<1, 2>())
       .def(py::init([](std::vector<std::pair<std::string, std::string>> hdrs,
                        py::bytes payload) {
              std::unordered_multimap<std::string, std::string> hdrs_map;
@@ -116,11 +115,11 @@ PYBIND11_MODULE(alephzero_bindings, m) {
              }
              return a0::Packet(std::move(hdrs_map), as_string_view(payload), a0::ref);
            }),
-           py::keep_alive<0, 2>())
+           py::keep_alive<1, 3>())
       .def(py::init([](py::str payload) {
              return a0::Packet(as_string_view(payload), a0::ref);
            }),
-           py::keep_alive<0, 1>())
+           py::keep_alive<1, 2>())
       .def(py::init([](std::vector<std::pair<std::string, std::string>> hdrs, py::str payload) {
              std::unordered_multimap<std::string, std::string> hdrs_map;
              for (auto& hdr : hdrs) {
@@ -128,7 +127,7 @@ PYBIND11_MODULE(alephzero_bindings, m) {
              }
              return a0::Packet(std::move(hdrs_map), as_string_view(payload), a0::ref);
            }),
-           py::keep_alive<0, 2>())
+           py::keep_alive<1, 3>())
       .def_property_readonly("id", &a0::Packet::id)
       .def_property_readonly("headers", [](a0::Packet* self) {
         std::vector<std::pair<std::string, std::string>> ret;
@@ -223,6 +222,7 @@ PYBIND11_MODULE(alephzero_bindings, m) {
       .def("alloc", &a0::TransportLocked::alloc)
       .def("alloc_evicts", &a0::TransportLocked::alloc_evicts)
       .def("commit", &a0::TransportLocked::commit)
+      .def("clear", &a0::TransportLocked::clear)
       .def("wait",
            &a0::TransportLocked::wait,
            py::call_guard<py::gil_scoped_release>(),
@@ -262,19 +262,39 @@ PYBIND11_MODULE(alephzero_bindings, m) {
       .def("push", &a0::Writer::push)
       .def("wrap", &a0::Writer::wrap);
 
-  py::enum_<a0::ReaderInit>(m, "ReaderInit")
-      .value("INIT_OLDEST", A0_INIT_OLDEST)
-      .value("INIT_MOST_RECENT", A0_INIT_MOST_RECENT)
-      .value("INIT_AWAIT_NEW", A0_INIT_AWAIT_NEW)
-      .export_values();
+  auto reader_py = py::class_<a0::Reader, nogil_holder<a0::Reader>>(m, "Reader");
 
-  py::enum_<a0::ReaderIter>(m, "ReaderIter")
-      .value("ITER_NEXT", A0_ITER_NEXT)
-      .value("ITER_NEWEST", A0_ITER_NEWEST)
-      .export_values();
+  auto reader_init_py = py::enum_<a0::Reader::Init>(m, "ReaderInit")
+      .value("OLDEST", a0::INIT_OLDEST)
+      .value("MOST_RECENT", a0::INIT_MOST_RECENT)
+      .value("AWAIT_NEW", a0::INIT_AWAIT_NEW);
+  reader_py.attr("Init") = reader_init_py;
+  m.attr("INIT_OLDEST") = reader_init_py.attr("OLDEST");
+  m.attr("INIT_MOST_RECENT") = reader_init_py.attr("MOST_RECENT");
+  m.attr("INIT_AWAIT_NEW") = reader_init_py.attr("AWAIT_NEW");
+
+  auto reader_iter_py = py::enum_<a0::Reader::Iter>(m, "ReaderIter")
+      .value("NEXT", a0::ITER_NEXT)
+      .value("NEWEST", a0::ITER_NEWEST);
+  reader_py.attr("Iter") = reader_iter_py;
+  m.attr("ITER_NEXT") = reader_iter_py.attr("NEXT");
+  m.attr("ITER_NEWEST") = reader_iter_py.attr("NEWEST");
+
+  py::class_<a0::Reader::Options>(reader_py, "Options")
+      .def(py::init<>())
+      .def(py::init<a0::Reader::Init>())
+      .def(py::init<a0::Reader::Iter>())
+      .def(py::init<a0::Reader::Init, a0::Reader::Iter>())
+      .def_property_readonly_static("DEFAULT", [](py::object) { return a0::Reader::Options::DEFAULT; })
+      .def_readwrite("init", &a0::Reader::Options::init)
+      .def_readwrite("iter", &a0::Reader::Options::iter);
 
   py::class_<a0::ReaderSyncZeroCopy>(m, "ReaderSyncZeroCopy")
-      .def(py::init<a0::Arena, a0::ReaderInit, a0::ReaderIter>())
+      .def(py::init<a0::Arena>())
+      .def(py::init<a0::Arena, a0::Reader::Options>())
+      .def(py::init<a0::Arena, a0::Reader::Init>())
+      .def(py::init<a0::Arena, a0::Reader::Iter>())
+      .def(py::init<a0::Arena, a0::Reader::Init, a0::Reader::Iter>())
       .def("can_read", &a0::ReaderSyncZeroCopy::can_read)
       .def("read", &a0::ReaderSyncZeroCopy::read)
       .def("read_blocking",
@@ -299,7 +319,11 @@ PYBIND11_MODULE(alephzero_bindings, m) {
           py::arg("timeout"));
 
   py::class_<a0::ReaderSync>(m, "ReaderSync")
-      .def(py::init<a0::Arena, a0::ReaderInit, a0::ReaderIter>())
+      .def(py::init<a0::Arena>())
+      .def(py::init<a0::Arena, a0::Reader::Options>())
+      .def(py::init<a0::Arena, a0::Reader::Init>())
+      .def(py::init<a0::Arena, a0::Reader::Iter>())
+      .def(py::init<a0::Arena, a0::Reader::Init, a0::Reader::Iter>())
       .def("can_read", &a0::ReaderSync::can_read)
       .def("read", &a0::ReaderSync::read)
       .def("read_blocking",
@@ -317,14 +341,36 @@ PYBIND11_MODULE(alephzero_bindings, m) {
 
   py::class_<a0::ReaderZeroCopy, nogil_holder<a0::ReaderZeroCopy>>(m, "ReaderZeroCopy")
       .def(py::init<a0::Arena,
-                    a0::ReaderInit,
-                    a0::ReaderIter,
+                    std::function<void(a0::TransportLocked, a0::FlatPacket)>>())
+      .def(py::init<a0::Arena,
+                    a0::Reader::Options,
+                    std::function<void(a0::TransportLocked, a0::FlatPacket)>>())
+      .def(py::init<a0::Arena,
+                    a0::Reader::Init,
+                    std::function<void(a0::TransportLocked, a0::FlatPacket)>>())
+      .def(py::init<a0::Arena,
+                    a0::Reader::Iter,
+                    std::function<void(a0::TransportLocked, a0::FlatPacket)>>())
+      .def(py::init<a0::Arena,
+                    a0::Reader::Init,
+                    a0::Reader::Iter,
                     std::function<void(a0::TransportLocked, a0::FlatPacket)>>());
 
-  py::class_<a0::Reader, nogil_holder<a0::Reader>>(m, "Reader")
+  reader_py
       .def(py::init<a0::Arena,
-                    a0::ReaderInit,
-                    a0::ReaderIter,
+                    std::function<void(a0::Packet)>>())
+      .def(py::init<a0::Arena,
+                    a0::Reader::Options,
+                    std::function<void(a0::Packet)>>())
+      .def(py::init<a0::Arena,
+                    a0::Reader::Init,
+                    std::function<void(a0::Packet)>>())
+      .def(py::init<a0::Arena,
+                    a0::Reader::Iter,
+                    std::function<void(a0::Packet)>>())
+      .def(py::init<a0::Arena,
+                    a0::Reader::Init,
+                    a0::Reader::Iter,
                     std::function<void(a0::Packet)>>());
 
   m.def("read_random_access", &a0::read_random_access);
@@ -341,7 +387,11 @@ PYBIND11_MODULE(alephzero_bindings, m) {
       .def("pub", py::overload_cast<a0::Packet>(&a0::Publisher::pub));
 
   py::class_<a0::SubscriberSync>(m, "SubscriberSync")
-      .def(py::init<a0::PubSubTopic, a0::ReaderInit, a0::ReaderIter>())
+      .def(py::init<a0::PubSubTopic>())
+      .def(py::init<a0::PubSubTopic, a0::Reader::Options>())
+      .def(py::init<a0::PubSubTopic, a0::Reader::Init>())
+      .def(py::init<a0::PubSubTopic, a0::Reader::Iter>())
+      .def(py::init<a0::PubSubTopic, a0::Reader::Init, a0::Reader::Iter>())
       .def("can_read", &a0::SubscriberSync::can_read)
       .def("read", &a0::SubscriberSync::read)
       .def("read_blocking",
@@ -359,8 +409,19 @@ PYBIND11_MODULE(alephzero_bindings, m) {
 
   py::class_<a0::Subscriber, nogil_holder<a0::Subscriber>>(m, "Subscriber")
       .def(py::init<a0::PubSubTopic,
-                    a0::ReaderInit,
-                    a0::ReaderIter,
+                    std::function<void(a0::Packet)>>())
+      .def(py::init<a0::PubSubTopic,
+                    a0::Reader::Options,
+                    std::function<void(a0::Packet)>>())
+      .def(py::init<a0::PubSubTopic,
+                    a0::Reader::Init,
+                    std::function<void(a0::Packet)>>())
+      .def(py::init<a0::PubSubTopic,
+                    a0::Reader::Iter,
+                    std::function<void(a0::Packet)>>())
+      .def(py::init<a0::PubSubTopic,
+                    a0::Reader::Init,
+                    a0::Reader::Iter,
                     std::function<void(a0::Packet)>>());
 
   py::class_<a0::RpcTopic>(m, "RpcTopic")
@@ -437,8 +498,7 @@ PYBIND11_MODULE(alephzero_bindings, m) {
       .value("INFO", a0::LogLevel::INFO)
       .value("DBG", a0::LogLevel::DBG)
       .value("MIN", a0::LogLevel::MIN)
-      .value("MAX", a0::LogLevel::MAX)
-      .export_values();
+      .value("MAX", a0::LogLevel::MAX);
 
   py::class_<a0::Logger>(m, "Logger")
       .def(py::init<a0::LogTopic>())
@@ -450,10 +510,40 @@ PYBIND11_MODULE(alephzero_bindings, m) {
       .def("dbg", py::overload_cast<a0::Packet>(&a0::Logger::dbg));
 
   py::class_<a0::LogListener, nogil_holder<a0::LogListener>>(m, "LogListener")
+      .def(py::init<a0::LogTopic,
+                    std::function<void(a0::Packet)>>())
+      .def(py::init<a0::LogTopic,
+                    a0::Reader::Options,
+                    std::function<void(a0::Packet)>>())
+      .def(py::init<a0::LogTopic,
+                    a0::Reader::Init,
+                    std::function<void(a0::Packet)>>())
+      .def(py::init<a0::LogTopic,
+                    a0::Reader::Iter,
+                    std::function<void(a0::Packet)>>())
+      .def(py::init<a0::LogTopic,
+                    a0::Reader::Init,
+                    a0::Reader::Iter,
+                    std::function<void(a0::Packet)>>())
       .def(py::init<a0::LogTopic,
                     a0::LogLevel,
-                    a0::ReaderInit,
-                    a0::ReaderIter,
+                    std::function<void(a0::Packet)>>())
+      .def(py::init<a0::LogTopic,
+                    a0::LogLevel,
+                    a0::Reader::Options,
+                    std::function<void(a0::Packet)>>())
+      .def(py::init<a0::LogTopic,
+                    a0::LogLevel,
+                    a0::Reader::Init,
+                    std::function<void(a0::Packet)>>())
+      .def(py::init<a0::LogTopic,
+                    a0::LogLevel,
+                    a0::Reader::Iter,
+                    std::function<void(a0::Packet)>>())
+      .def(py::init<a0::LogTopic,
+                    a0::LogLevel,
+                    a0::Reader::Init,
+                    a0::Reader::Iter,
                     std::function<void(a0::Packet)>>());
 
   py::class_<a0::CfgTopic>(m, "CfgTopic")
